@@ -2,7 +2,7 @@
 #
 # Creates the API documentation for igraph's Python interface using PyDoctor
 #
-# Usage: ./mkdoc.sh [--sync] [directory]
+# Usage: ./mkdoc.sh
 
 SCRIPTS_FOLDER=`dirname $0`
 
@@ -35,7 +35,7 @@ rm -rf "${SITE_PACKAGES_DIR}"/python_igraph*.egg
 rm -rf "${SITE_PACKAGES_DIR}"/python_igraph*.egg-link
 
 echo "Installing python-igraph in virtualenv..."
-rm -f dist/*.whl && .venv/bin/python setup.py bdist_wheel && .venv/bin/pip install dist/*.whl
+rm -f dist/*.whl && .venv/bin/python setup.py bdist_wheel && .venv/bin/pip install --force-reinstall dist/*.whl
 
 IGRAPHDIR=`.venv/bin/python3 -c 'import igraph, os; print(os.path.dirname(igraph.__file__))'`
 
@@ -50,5 +50,26 @@ echo "Generating HTML documentation..."
 
 # PDF not supported by PyDoctor
 
+DOC2DASH=`which doc2dash 2>/dev/null || true`
+if [ "x$DOC2DASH" != x ]; then
+    echo "Generating Dash docset..."
+    "$DOC2DASH" \
+        --online-redirect-url "https://igraph.org/python/doc/api" \
+        --name "python-igraph" \
+        -d "${DOC_API_FOLDER}" \
+	    -f \
+        "${DOC_API_FOLDER}/html"
+    DASH_READY=1
+else 
+    echo "WARNING: doc2dash not installed, skipping Dash docset generation."
+    DASH_READY=0
+fi
+
+echo ""
+echo "HTML API documentation generated in ${DOC_API_FOLDER}/html"
+if [ "x${DASH_READY}" = x1 ]; then
+    echo "Dash docset generated in ${DOC_API_FOLDER}/python-igraph.docset"
+fi
+
 cd "$PWD"
 
