@@ -4,7 +4,7 @@ Drawing routines to draw graphs.
 This module contains routines to draw graphs on:
 
   - Cairo surfaces (L{DefaultGraphDrawer})
-  - UbiGraph displays (L{UbiGraphDrawer}, see U{http://ubietylab.net/ubigraph})
+  - Matplotlib axes (L{MatplotlibGraphDrawer})
 
 It also contains routines to send an igraph graph directly to
 (U{Cytoscape<http://www.cytoscape.org>}) using the
@@ -13,8 +13,6 @@ L{CytoscapeGraphDrawer}. L{CytoscapeGraphDrawer} can also fetch the current
 network from Cytoscape and convert it to igraph format.
 """
 
-from collections import defaultdict
-
 from math import atan2, cos, pi, sin, tan, sqrt
 from warnings import warn
 
@@ -34,23 +32,20 @@ from igraph.drawing.utils import find_cairo, Point
 from igraph.drawing.vertex import DefaultVertexDrawer
 from igraph.layout import Layout
 
-__all__ = ("DefaultGraphDrawer", "UbiGraphDrawer", "CytoscapeGraphDrawer")
-__license__ = "GPL"
+__all__ = (
+    "DefaultGraphDrawer", "MatplotlibGraphDrawer", "CytoscapeGraphDrawer",
+    "UbiGraphDrawer"
+)
 
 cairo = find_cairo()
 
 #####################################################################
 
 
-# pylint: disable-msg=R0903
-# R0903: too few public methods
 class AbstractGraphDrawer(AbstractDrawer):
     """Abstract class that serves as a base class for anything that
     draws an igraph.Graph."""
 
-    # pylint: disable-msg=W0221
-    # W0221: argument number differs from overridden method
-    # E1101: Module 'cairo' has no 'foo' member - of course it does :)
     def draw(self, graph, *args, **kwds):
         """Abstract method, must be implemented in derived classes."""
         raise NotImplementedError("abstract class")
@@ -223,10 +218,6 @@ class DefaultGraphDrawer(AbstractCairoGraphDrawer):
 
         return vertex_order
 
-    # pylint: disable-msg=W0142,W0221,E1101
-    # W0142: Used * or ** magic
-    # W0221: argument number differs from overridden method
-    # E1101: Module 'cairo' has no 'foo' member - of course it does :)
     def draw(self, graph, palette, *args, **kwds):
         # Some abbreviations for sake of simplicity
         directed = graph.is_directed()
@@ -442,7 +433,7 @@ class DefaultGraphDrawer(AbstractCairoGraphDrawer):
                     # Intersection at bottom edge of label
                     try:
                         cx += h / tan(alpha)
-                    except:
+                    except Exception:
                         pass  # tan(alpha) == inf
                     cy -= h
                 elif alpha > gamma and alpha <= gamma + 2 * beta:
@@ -453,7 +444,7 @@ class DefaultGraphDrawer(AbstractCairoGraphDrawer):
                     # Intersection at top edge of label
                     try:
                         cx -= h / tan(alpha)
-                    except:
+                    except Exception:
                         pass  # tan(alpha) == inf
                     cy += h
                 # Draw the label
@@ -542,6 +533,11 @@ class UbiGraphDrawer(AbstractXMLRPCDrawer, AbstractGraphDrawer):
     The drawer also has two attributes, C{vertex_defaults} and
     C{edge_defaults}. These are dictionaries that can be used to
     set default values for the vertex/edge attributes in Ubigraph.
+
+    @deprecated: UbiGraph has not received updates since 2008 and is now not
+        available for download (at least not from the official sources).
+        The UbiGraph graph drawer will be removed from python-igraph in
+        0.10.0.
     """
 
     def __init__(self, url="http://localhost:20738/RPC2"):
@@ -551,6 +547,11 @@ class UbiGraphDrawer(AbstractXMLRPCDrawer, AbstractGraphDrawer):
         self.vertex_defaults = dict(color="#ff0000", shape="cube", size=1.0)
         self.edge_defaults = dict(color="#ffffff", width=1.0)
 
+        warn(
+            "UbiGraphDrawer is deprecated from python-igraph 0.9.4",
+            DeprecationWarning
+        )
+
     def draw(self, graph, *args, **kwds):
         """Draws the given graph on an UbiGraph display.
 
@@ -903,10 +904,11 @@ class GephiGraphStreamingDrawer(AbstractGraphDrawer):
 
     The Gephi graph streaming format is a simple JSON-based format that can be used
     to post mutations to a graph (i.e. node and edge additions, removals and updates)
-    to a remote component. For instance, one can open up Gephi (U{http://www.gephi.org}),
-    install the Gephi graph streaming plugin and then send a graph from igraph
-    straight into the Gephi window by using C{GephiGraphStreamingDrawer} with the
-    appropriate URL where Gephi is listening.
+    to a remote component. For instance, one can open up Gephi
+    (U{http://www.gephi.org}), install the Gephi graph streaming plugin and then
+    send a graph from igraph straight into the Gephi window by using
+    C{GephiGraphStreamingDrawer} with the appropriate URL where Gephi is
+    listening.
 
     The C{connection} property exposes the L{GephiConnection} that the drawer
     uses. The drawer also has a property called C{streamer} which exposes the underlying
@@ -977,6 +979,7 @@ class MatplotlibGraphDrawer(AbstractGraphDrawer):
 
     def draw(self, graph, *args, **kwds):
         # NOTE: matplotlib has numpy as a dependency, so we can use it in here
+        from collections import defaultdict
         import matplotlib as mpl
         import matplotlib.markers as mmarkers
         from matplotlib.path import Path
@@ -984,6 +987,9 @@ class MatplotlibGraphDrawer(AbstractGraphDrawer):
         from matplotlib.patches import ArrowStyle
         import numpy as np
 
+        # Deferred import to avoid a cycle in the import graph
+        from igraph.clustering import VertexClustering, VertexCover
+
         def shrink_vertex(ax, aux, vcoord, vsize_squared):
             """Shrink edge by vertex size"""
             aux_display, vcoord_display = ax.transData.transform([aux, vcoord])
@@ -1010,6 +1016,14 @@ class MatplotlibGraphDrawer(AbstractGraphDrawer):
 
         # FIXME: deal with unnamed *args
 
+        # graph is not necessarily a graph, it can be a VertexClustering. If so
+        # extract the graph. The clustering itself can be overridden using
+        # the "mark_groups" option
+        clustering = None
+        if isinstance(graph, (VertexClustering, VertexCover)):
+            clustering = graph
+            graph = clustering.graph
+
         # Get layout
         layout = kwds.get("layout", graph.layout())
         if isinstance(layout, str):
@@ -1018,6 +1032,104 @@ class MatplotlibGraphDrawer(AbstractGraphDrawer):
         # Vertex coordinates
         vcoord = layout.coords
 
+        # mark groups: the used data structure is eventually the dict option:
+        # tuples of vertex indices as the keys, colors as the values. We
+        # convert other formats into that one here
+        if "mark_groups" not in kwds:
+            kwds["mark_groups"] = False
+        if kwds["mark_groups"] is False:
+            pass
+        elif (kwds["mark_groups"] is True) and (clustering is not None):
+            pass
+        elif isinstance(kwds["mark_groups"], (VertexClustering, VertexCover)):
+            if clustering is not None:
+                raise ValueError(
+                    "mark_groups cannot override a clustering with another"
+                )
+            else:
+                clustering = kwds["mark_groups"]
+                kwds["mark_groups"] = True
+        else:
+            try:
+                mg_iter = iter(kwds["mark_groups"])
+            except TypeError:
+                raise TypeError("mark_groups is not in the right format")
+            kwds["mark_groups"] = dict(mg_iter)
+
+        # If a clustering is set and marks are requested but without a specific
+        # colormap, make the colormap
+
+        # Two things need coloring: vertices and groups/clusters (polygon)
+        # The coloring needs to be coordinated between the two.
+        if clustering is not None:
+            # If mark_groups is a dict, we don't need a default color dict, we
+            # can just use the mark_groups dict. If mark_groups is False and
+            # vertex_color is set, we don't need either because the colors are
+            # already fully specified. In all other cases, we need a default
+            # color dict.
+            if isinstance(kwds["mark_groups"], dict):
+                group_colordict = kwds["mark_groups"]
+            elif (kwds["mark_groups"] is False) and ("vertex_color" in kwds):
+                pass
+            else:
+                membership = clustering.membership
+                if isinstance(clustering, VertexCover):
+                    membership = [x[0] for x in membership]
+                clusters = sorted(set(membership))
+                n_colors = len(clusters)
+                cmap = mpl.cm.get_cmap("viridis")
+                colors = [cmap(1.0 * i / n_colors) for i in range(n_colors)]
+                cluster_colordict = {g: c for g, c in zip(clusters, colors)}
+
+                # mark_groups if not explicitly marked
+                group_colordict = defaultdict(list)
+                for i, g in enumerate(membership):
+                    color = cluster_colordict[g]
+                    group_colordict[color].append(i)
+                del cluster_colordict
+                # Invert keys and values
+                group_colordict = {tuple(v): k for k, v in group_colordict.items()}
+
+            # If mark_groups is set but not defined, make a default colormap
+            if kwds["mark_groups"] is True:
+                kwds["mark_groups"] = group_colordict
+
+            if "vertex_color" not in kwds:
+                kwds["vertex_color"] = ['none' for m in membership]
+                for group_vs, color in group_colordict.items():
+                    for i in group_vs:
+                        kwds["vertex_color"][i] = color
+
+            # Now mark_groups is either a dict or False
+            # If vertex_color is not set, we can rely on mark_groups if a dict,
+            # else we need to make up the same colormap as if we were requested groups
+            if "vertex_color" not in kwds:
+                if isinstance(kwds["mark_groups"], dict):
+                    membership = clustering.membership
+                    if isinstance(clustering, VertexCover):
+                        membership = [x[0] for x in membership]
+
+        # Mark groups
+        if "mark_groups" in kwds:
+            for idx, color in kwds["mark_groups"].items():
+                points = [vcoord[i] for i in idx]
+                vertices = np.asarray(convex_hull(points, coords=True))
+                # 15% expansion
+                vertices += 0.15 * (vertices - vertices.mean(axis=0))
+
+                # NOTE: we could include a corner cutting algorithm close to
+                # the vertices (e.g. Chaikin) for beautification, or a corner
+                # radius like it's done in the Cairo interface
+                polygon = mpl.patches.Polygon(
+                    vertices,
+                    facecolor=color,
+                    alpha=0.3,
+                    zorder=0,
+                    edgecolor=color,
+                    lw=2,
+                )
+                ax.add_artist(polygon)
+
         # Vertex properties
         nv = graph.vcount()
 
@@ -1101,7 +1213,13 @@ class MatplotlibGraphDrawer(AbstractGraphDrawer):
             if default is True:
                 default = 0.5
             default = float(default)
-            kwds["edge_curved"] = autocurve(graph, attribute=None, default=default)
+            ecurved = autocurve(graph, attribute=None, default=default)
+        elif "edge_curved" in kwds:
+            ecurved = kwds["edge_curved"]
+        elif "curved" in graph.edge_attributes():
+            ecurved = graph.es["curved"]
+        else:
+            ecurved = [0] * ne
 
         # Arrow style for directed and undirected graphs
         if graph.is_directed():
@@ -1115,7 +1233,6 @@ class MatplotlibGraphDrawer(AbstractGraphDrawer):
 
         # Edge coordinates and curvature
         nloops = [0 for x in range(ne)]
-        has_curved = "curved" in graph.es.attributes()
         arrows = []
         for ie, edge in enumerate(graph.es):
             src, tgt = edge.source, edge.target
@@ -1195,14 +1312,14 @@ class MatplotlibGraphDrawer(AbstractGraphDrawer):
                 )
 
             else:
-                curved = edge["curved"] if has_curved else False
+                curved = ecurved[ie]
                 if curved:
-                    aux1 = (2 * x1 + x2) / 3.0 - edge.curved * 0.5 * (y2 - y1), (
+                    aux1 = (2 * x1 + x2) / 3.0 - curved * 0.5 * (y2 - y1), (
                         2 * y1 + y2
-                    ) / 3.0 + edge.curved * 0.5 * (x2 - x1)
-                    aux2 = (x1 + 2 * x2) / 3.0 - edge.curved * 0.5 * (y2 - y1), (
+                    ) / 3.0 + curved * 0.5 * (x2 - x1)
+                    aux2 = (x1 + 2 * x2) / 3.0 - curved * 0.5 * (y2 - y1), (
                         y1 + 2 * y2
-                    ) / 3.0 + edge.curved * 0.5 * (x2 - x1)
+                    ) / 3.0 + curved * 0.5 * (x2 - x1)
                     start = shrink_vertex(ax, aux1, (x1, y1), vsizes[src])
                     end = shrink_vertex(ax, aux2, (x2, y2), vsizes[tgt])
 
